//////////////////////////////////////////////////////////////////////////////
// main.cpp                                                                 //
//////////////////////////////////////////////////////////////////////////////
/*
	GbaTool by Rafael Vuijk (aka Dark Fader)
	
	Features:
	- UART upload
		SIO will not be reset
	- Multiboot menu/loader
		use up/down/A buttons to choose
		press L+R prior if you don't want to reset SIO (for debugging)

	Notes:
	- don't use multiple interrupts
	
	History:
	- v1.03 Halt in main loop
	- v1.02 recompile with latest conio & background color :P
	- v1.01 some minor fixes
	- v1.00	took GbaSioTool and added nice multiboot selection menu
*/

//////////////////////////////////////////////////////////////////////////////
// Includes                                                                 //
//////////////////////////////////////////////////////////////////////////////
#include <GBA.h>
#include <stdio.h>
#include <debug.h>
#include <stdlib.h>
#include <string.h>
#include <conio.h>
#include "res.h"

//////////////////////////////////////////////////////////////////////////////
// Defines                                                                  //
//////////////////////////////////////////////////////////////////////////////
#define VER					"1.03"
#define MAX_CART_SIZE		(256/8*1024*1024)
#define MIN_MB_SIZE			256					// minimum gbafix padded size

//////////////////////////////////////////////////////////////////////////////
// Typedefs                                                                 //
//////////////////////////////////////////////////////////////////////////////
struct RomData
{
	const u8	*address;
	char		title[14];
};

//////////////////////////////////////////////////////////////////////////////
// Variables                                                                //
//////////////////////////////////////////////////////////////////////////////
RomData roms[256];
vKeyData keyData;
int romcount;
int select = 0;
bool rom_debug = false;
//volatile int timeout = 0;	// vblank timeout counter

//////////////////////////////////////////////////////////////////////////////
// Prototypes                                                               //
//////////////////////////////////////////////////////////////////////////////
void SioCommandIntr();
void SioRegWriteIntr();

//////////////////////////////////////////////////////////////////////////////
// CartridgeSize                                                            //
//////////////////////////////////////////////////////////////////////////////
u32 CartridgeSize()
{
	// mirror check
	u32 size = 256;
	while (size <= MAX_CART_SIZE)
	{
		u32 *a = (u32 *)(ROM_BANK0);
		u32 *b = (u32 *)(ROM_BANK0 + size);
		int i;
		for (i=0xC0; i; i--)
		{
			if (*a++ != *b++) break;	// not a mirror
		}
		if (!i) break;		// mirror
		size <<= 1;
	}
	return size;
}

//////////////////////////////////////////////////////////////////////////////
// Boot                                                                     //
//////////////////////////////////////////////////////////////////////////////
void Boot(const u8 *program)
{
	rIME = 0;
	DmaCopy(3, program, EX_WRAM, (EX_WRAM_END-EX_WRAM), 32);
	rSIOCNT = SIO_UART_MODE | SIO_115200_BPS | SIO_CTS_ENABLE | SIO_UART_8BIT | SIO_TRANS_ENABLE | SIO_RECV_ENABLE;
	SoftResetExram(rom_debug ? RESET_ALL_FLAG - RESET_REG_SIO_FLAG : RESET_ALL_FLAG);
}

//////////////////////////////////////////////////////////////////////////////
// VBlankIntr                                                               //
//////////////////////////////////////////////////////////////////////////////
void VBlankIntr()
{
	//if (timeout > 0) if (--timeout == 0) SoftResetRom(RESET_ALL_FLAG);
	
	// process keys
	(KeyData &)keyData = (KeyData &)rKeyData;
	//if (!keyData.Start) quit = true;		// reset on start button or escape from console
	
	rINTR_CHECK_BUF = V_BLANK_INTR_FLAG;     // Set V Blank interrupt check
}

//////////////////////////////////////////////////////////////////////////////
// SioCommandIntr                                                           //
//////////////////////////////////////////////////////////////////////////////
void SioCommandIntr()
{
	bg_palette[0] = RGB(31,0,0);
	//rIE &=~ SIO_INTR_FLAG;
	//rSioUartCnt.IF_Enable = 0;
	
	if (!rSioUartCnt.RecvDataEmpty)
	{
		switch (RecvByte())
		{
			case 27:	// reset or something
			case '?':	// check for command mode
			{
				bg_palette[0] = RGB(0,31,0);
				SendByte('!');
				break;
			}
			
			case 'w':	// write int to memory
			{
				int *p = (int *)RecvInt();
				*p = RecvInt();
				break;
			}
			
			case 'i':	// 16 bits register writes
			{
				iSioIntr = SioRegWriteIntr;
				break;
			}
			
			case 'r':	// read int from memory
			{
				int *p = (int *)RecvInt();
				SendInt(*p);
				break;
			}
			
			case 'u':	// upload block to EX_WRAM
			{
				//rIE &=~ V_BLANK_INTR_FLAG;
				//rSioUartCnt.Fifo_Enable = 1;
				char *p = (char *)EX_WRAM;//(int *)RecvInt();
				int size = RecvInt();
				//clrscr(); cprintf("Downloading %d bytes...\n", size);
				//timeout = 60;
				//for (int i=0; i<size; i++)
				while (size--)
				{
					//rSioUartCnt.Recv_Enable = 1;
					int c = RecvByte();
					//rSioUartCnt.Recv_Enable = 0;
					//if (c < 0) cprintf("\rERROR!\n");
					*p++ = c;
					//if ((i & 255) == 0) { cprintf("\r%d %%", i*100/size); timeout = 60; }
					//{ cprintf("\r%d", i); timeout = 60; }
				}
				//cprintf("100 %%... download complete\n");
				//timeout = 0;
				//rSioUartCnt.Fifo_Enable = 0;
				//rIE |= V_BLANK_INTR_FLAG;
				break;
			}
			
			case 'x':	// execute code at 0x02000000	// 256kb
			{
				rSIOCNT = SIO_UART_MODE | SIO_115200_BPS | SIO_CTS_ENABLE | SIO_UART_8BIT | SIO_TRANS_ENABLE | SIO_RECV_ENABLE;// | SIO_IF_ENABLE;
				SoftResetExram(RESET_ALL_FLAG - RESET_REG_SIO_FLAG);	// leave debugging enabled
				break;
			}
			
			case 'a':	// update affine data
			{
				ObjAffineSrcData ad;
				ad.RatioX = RecvInt();	// :8
				ad.RatioY = RecvInt();	// :8
				ad.Theta = RecvInt();		// 16:8
				ObjAffineSet(&ad, (void *)RecvInt(), 1, 8);
				break;
			}
			
			case '2':	// bkg2 affine
			{
				BgAffineSrcData ad;
				ad.SrcCenterX = RecvInt();	// :8
	 			ad.SrcCenterY = RecvInt();	// :8
				ad.DispCenterX = RecvInt();
				ad.DispCenterY = RecvInt();
				ad.RatioX = RecvInt();	// :8
				ad.RatioY = RecvInt();	// :8
				BgAffineSet(&ad, (BgAffineDestData *)&rBg2Affine, 0);
				break;
			}
	
			case '3':	// bkg2 affine
			{
				BgAffineSrcData ad;
				ad.SrcCenterX = RecvInt();	// :8
	 			ad.SrcCenterY = RecvInt();	// :8
				ad.DispCenterX = RecvInt();
				ad.DispCenterY = RecvInt();
				ad.RatioX = RecvInt();	// :8
				ad.RatioY = RecvInt();	// :8
				BgAffineSet(&ad, (BgAffineDestData *)&rBg3Affine, 0);
				break;
			}
			
			case 'p':	// set palette
			{
				DmaCopy(3, EX_WRAM, bg_palette, sizeof(Palette), 16);
				break;
			}
	
			case 'b':	// set bitmap
			{
				DmaCopy(3, EX_WRAM, bg_bitmap0, sizeof(Bitmap4), 16);
				break;
			}
			
			case 'z':	// un LZ-77 bitmap
			{
				LZ77UnCompVram((void *)EX_WRAM, (void *)bg_bitmap0);
				break;
			}
			
			default:	// unknown command
			{
				SendByte('?');
				break;
			}
		}
	}

	//rIE |= SIO_INTR_FLAG;
	//rSioUartCnt.IF_Enable = 1;
}

//////////////////////////////////////////////////////////////////////////////
// SioRegWriteIntr                                                          //
//////////////////////////////////////////////////////////////////////////////
void SioRegWriteIntr()
{
	if (!rSioUartCnt.RecvDataEmpty)
	{
		// 16 bits register writes
		u8 addr = RecvByte();
		if (!addr) { iSioIntr = SioCommandIntr; }	// if 0, return to command mode
		u16 *address = (u16 *)(REG_BASE + (addr<<1));
		u32 data = RecvByte();
		data += RecvByte() << 8;
		*address = data;
	}
}

//////////////////////////////////////////////////////////////////////////////
// ScanRoms                                                                 //
//////////////////////////////////////////////////////////////////////////////
int ScanRoms()
{
	u32 cartSize = CartridgeSize();
	romcount = 0;
	for (u32 p=ROM_BANK0+MIN_MB_SIZE/*skip first*/; p<ROM_BANK0+cartSize; p+=MIN_MB_SIZE)
	{
		if (*(u16 *)(p+2) == 0xEA00)//002E)	// branch
		if (*(u32 *)(p+4) == 0x51AEFF24)	// logo
		if (*(u32 *)(p+8) == 0x21A29A69)
		if (*(u32 *)(p+12) == 0x0A82843D)
		if (*(u32 *)(p+16) == 0xAD09E484)
		{
			roms[romcount].address = (u8 *)p;
			strncpy(roms[romcount].title, (char *)(p + 0xA0), 12);
			romcount++;
		}
	}
	return romcount;
}

//////////////////////////////////////////////////////////////////////////////
// ShowRoms                                                                 //
//////////////////////////////////////////////////////////////////////////////
void ShowRoms()
{
	int y = wherey();
	textcolor(7);
	for (int i=0; i<romcount; i++)
	{
		cprintf("%c %03d - %s\n", (i == select) ? '>' : ' ', i, roms[i].title);
	}
	gotoxy(0,y);
}

//////////////////////////////////////////////////////////////////////////////
// Init                                                                     //
//////////////////////////////////////////////////////////////////////////////
void Init()
{
	// init video
	rDISPCNT = DISP_MODE_4 | DISP_BG2_ON;		// 256 color bitmapped mode
	rSTAT = STAT_V_BLANK_IF_ENABLE;
	const BgAffineDestData bgAffineReset = {256,0,0,256,0,0};
	(BgAffineDestData &)rBg2Affine = bgAffineReset;
	iVBlankIntr = VBlankIntr; rIME = 0; rIE |= V_BLANK_INTR_FLAG; rIME = 1;

	// init UART
	InitUART();
	iSioIntr = SioCommandIntr;
	rIME = 0; rIE |= SIO_INTR_FLAG; rIME = 1;	

	// init keypad
	(KeyData &)keyData = (KeyData &)rKeyData;
	iKeyIntr = DummyIntr; rIME = 0; rIE |= KEY_INTR_FLAG; rIME = 1;
	rKEYCNT = /*KEY_IF_ENABLE |*/ KEY_OR_INTR | ALL_KEY_MASK;
}

//////////////////////////////////////////////////////////////////////////////
// main                                                                     //
//////////////////////////////////////////////////////////////////////////////
int main(int argc, char argv[])
{
	printf("GbaTool\n");

	Init();

	// scan roms
	textmode(CO60);
	textbackground(BLUE);
	clrscr();
	textcolor(WHITE); cputs("GbaTool v"VER" by Rafael Vuijk (aka Dark Fader)\n");	
	ScanRoms();
		romcount = 3;
	textcolor(YELLOW);
		cprintf("%08X %08X %08X %08X %08X\n", __putchar, __getchar, getcharBuf, getcharBufIn, getcharBufOut);
	cprintf("%d MB-rom(s) found. Select or upload to run.\n\n", romcount);
	ShowRoms();

	// main loop
	while (1)
	{
		if (!keyData.L && !keyData.R)	// debug for attached programs?
		{
			rom_debug = true;
		}

		if (!keyData.Plus_U)
		{
			if (select > 0) select--;
			ShowRoms();
			for (int i=0; i<10; i++) { VBlankIntrWait(); if (keyData.Plus_U) break; }
		}
	
		if (!keyData.Plus_D)
		{
			if (select < romcount-1) select++;
			ShowRoms();
			for (int i=0; i<10; i++) { VBlankIntrWait(); if (keyData.Plus_D) break; }
		}
	
		if (!keyData.A)
		{
			Boot(roms[select].address);
		}

		//Halt();
		VBlankIntrWait();
		
		bg_palette[0] = (bg_palette[0] + RGB(0,0,1)) & RGB(0,0,31);
	}
}
